<?php

/**
 * @file
 * Contains definations for Drush commands and callbacks.
 */

use Drupal\node\Entity\Node;

/**
 * Implements hook_drush_command().
 */
function cleantaxonomy_drush_command() {
  $commands['cleantaxonomy-replace-callback'] = [
    'description' => 'Replace taxonomy terms using cleantaxonomy',
    'aliases' => ['ct-replace'],
    'arguments' => [
      'term1' => "Name of the old term to be replaced.",
      'term2' => "Name of the new term.",
    ],
    'examples' => [
      'drush ct-replace term1 term2' => "replaces the term1 with term2",
    ],
  ];
  return $commands;
}

/**
 * Drush command logic for command ct-replace.
 */
function drush_cleantaxonomy_replace_callback($arg1, $arg2) {
  $name1 = $arg1;
  $name2 = $arg2;
  if ($name1 === $name2) {
    drush_set_error("Both terms are same. Please choose different terms.");
    return;
  }
  $term_from = \Drupal::database()->select('taxonomy_term_field_data', 'x')
    ->fields('x', ['tid']);
  $term_from->condition('x.name', $name1);
  $data = $term_from->execute();
  $term_from = $data->fetchAll(\PDO::FETCH_ASSOC);
  $term_from = $term_from[0]['tid'];

  $term2_obj = \Drupal::entityTypeManager()->getStorage('taxonomy_term')->loadByProperties(['name' => $name2]);

  if (empty($term_from) || empty($term2_obj)) {
    drush_set_error("Terms not found! Please enter correct terms.");
  }
  else {
    $term2_object = reset($term2_obj);
    $term_to = $term2_object->id();

    // Retrieving nids having taxonomy term 'from'.
    $duplicate = \Drupal::database()->select('taxonomy_index', 'x')
      ->fields('x', ['nid']);
    $duplicate->condition('x.tid', $term_from);
    $data = $duplicate->execute();
    $nodes_with_term_from = $data->fetchAll(\PDO::FETCH_ASSOC);

    $entityManager = \Drupal::service('entity_field.manager');
    foreach ($nodes_with_term_from as $value) {
      $node = Node::load($value['nid']);
      $bundle = $node->bundle();
      $fields = $entityManager->getFieldDefinitions('node', $bundle);
      foreach ($node as $key => $values) {
        if (substr($key, 0, 5) == 'field') {
          if ($fields[$key]->getSettings()['target_type'] == 'taxonomy_term') {
            $field_machine_name = $fields[$key]->getName();
            $taxonomy_terms = $node->get($field_machine_name)->getValue();
            $updated_terms = [];
            foreach ($taxonomy_terms as $target_id) {
              if ($target_id['target_id'] == $term_from) {
                $updated_terms[] = ['target_id' => $term_to];
              }
              else {
                $updated_terms[] = ['target_id' => $target_id['target_id']];
              }
            }
            $updated_terms = array_unique($updated_terms);
            $node->set($field_machine_name, $updated_terms);
            $node->save();
          }
        }
      }
    }
    drush_print_r("Replaced Successfully.");
  }
}
