<?php

namespace Drupal\cleantaxonomy\Controller;

use Drupal\Component\Utility\Xss;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\Element\EntityAutocomplete;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Defines a route controller for watches autocomplete form elements.
 */
class TaxonomyAutoCompleteController extends ControllerBase {

  /**
   * Entity Type Manager Interface.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    // Instantiates this form class.
    return new static(
      $container->get('entity_type.manager')
    );
  }

  /**
   * Handler for autocomplete request for Taxonomy Terms.
   */
  public function handleTaxonomyAutocomplete(Request $request, $taxonomy_term_vid = NULL) {
    $results = [];
    // Get the typed string from the URL, if it exists.
    $input = $request->query->get('q');
    if (!$input) {
      return new JsonResponse($results);
    }
    $input = Xss::filter($input);
    // dpm($input);
    $query = $this->entityTypeManager->getStorage('taxonomy_term')->getQuery()->accessCheck(FALSE)
      ->condition('name', $input, 'CONTAINS')
      ->condition('vid', $taxonomy_term_vid)
      ->sort('name')
      ->range(0, 10);

    $ids = $query->execute();
    $ids = array_keys($ids);

    $terms = $ids ? $this->entityTypeManager->getStorage('taxonomy_term')->loadMultiple($ids) : [];
    foreach ($terms as $term) {
      $value = explode(" ", EntityAutocomplete::getEntityLabels([$term]))[0];
      $results[] = [
        'value' => $value,
        'label' => $term->getName(),
      ];
    }
    return new JsonResponse($results);
  }

}
